/* FIT.C : Copyright 1990 Turgut Kalfaoglu, 1378 Sokak 8/10,Izmir, Turkey.

   This program will attempt to fit as many files as it can onto a given disk.
   It does this by first copying a large file, and then selecting the file
   that would best fit in the remaining space. The best fit search is not
   'great' and there is room for improvement. Please send me your version
   so that we can distribute the 'best' version..

   I can be reached at <TURGUT@TREARN.BITNET>.
   Compilable under Microsoft C (R) 6.0, under OS/2
   Make sure to link it with SETARGV.OBJ
   Regards,  -Turgut Kalfaoglu  Jan 28th, 1991

   Usage:
     FIT <wildcard specification> destination_drive

   Examples:
     FIT *.EXE *.DOC \SOMEDIR\*.TXT A:
     FIT *.* B:\

*/
/* This version of FIT can only be compiled under OS/2, so do not remove
   the definition below */
#define os2

#ifdef os2
  #define INCL_DOS
  #include <os2.h>
#endif
#pragma optimize("t",off)
#include <io.h>
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <dos.h>
#include <ctype.h>
#include <string.h>
#include <limits.h>
#include <errno.h>
#include <stdio.h>
#include <malloc.h>
#include <stdlib.h>
#include <conio.h>
#include <process.h>
void disp_help(void);
long learn_drive(void);
void change_disks(void);
int pick_file(void);
unsigned load_file_info(int num,char *names[]);
int copyfile(unsigned int);
void fatal(char *);

/* This section is not used for this version */
#ifdef os2
    char *cmdproc = "CMD.EXE";
    char *os = "OS/2";
#else
    char *cmdproc = "COMMAND.COM";
    char *os = "DOS";
#endif

char *filename[1024];
unsigned long filesize[1024];
char destination; /* drive letter */
unsigned num_of_files; /* ..left to copy */
unsigned total_files;
unsigned long bytesfree;
unsigned short disk_empty; /* flag to state that it's a blank diskette */
unsigned short prompt=0; /* to pause or not when diskette is full */

main(int argc, char *argv[]) {
    int i;
    char blurb[255]; /* used for complaining about errors */

    if (argc<3) disp_help();
                             /* record filenames and sizes */
    total_files = num_of_files = load_file_info(argc,argv); 

    destination = toupper(argv[argc-1][0]); /* the drive letter, upcased */

    while (num_of_files > 0) {
        i=pick_file();
        if (i == -1) { /* if pick_file failed to pick a file.. */
            change_disks();
            continue; }
       if (copyfile(i) != 0) { /* if copyfile failed, complain and quit */
            strcpy(blurb,"Error copying ");
            strcat(blurb,filename[i]);
            fatal(blurb); }
        filename[i][0] = 0;
        num_of_files--;}
}

/* we choose a file among those that we have left, and return its number
*/

int pick_file() {
    int i;
    unsigned long leaves=ULONG_MAX;
    long diff; /* keeps the space that would remain after copying this
                  particular file, can be - */
    int best=-1;

    /* we try to find the file that would leave the least amount of space
    on disk */

    bytesfree = learn_drive();
    for (i=0;i<total_files;i++) {
        if (filename[i][0] == 0) continue; /* if already copied.. */

        /* is the file too large? (diskette empty and filesize is bigger
           than remaining space?) */
        if (disk_empty && (filesize[i]>bytesfree)) {
            printf("%s is larger than your diskette size - skipping\n",filename[i]);
            filename[i][0] = 0;
            num_of_files--;
            continue; }
        diff = bytesfree - filesize[i];
        /* if there is a file that's better, use that one instead */
        if (diff<0L || diff > leaves) continue; 
        best = i;
        leaves = bytesfree - filesize[i]; } /* our best pick would leave 
                                               'leave' bytes on disk */
    return (best); /* return index # of best pick */
}

/* we just prompt user here to insert a new diskette */
void change_disks() {
    char c;
/* This section is not yet implemented (never had time)    -tk
    if (prompt) {
      printf("Insert new diskette in drive %c:\n",destination);
      printf("Hit <D> for %s, CTRL-C to abort, or any other key to continue ",os);
      c = getche();
      c = toupper(c);
      printf("\n");
      if (c == 'D') {
        printf("Type EXIT when ready\n");
        spawnlp(P_WAIT,cmdproc,cmdproc,"spawnlp",NULL);
        printf("Insert new diskette in drive %c:\n",destination);
        printf("Hit any key to continue\n");
        c = getch(); }
    }
    else
*/
     {
        printf("Remove diskette in drive %c\n",destination);
        DosError(0); /* disable errors */
        while(learn_drive() != -1L); /* wait until they remove diskette */
        printf("Insert new diskette in drive %c\n",destination);
        while(learn_drive() == -1L); /* ..until they put in the new diskette! */
        DosError(1); /* enable crit.error handler */ }
}

/* here we copy a given file to destination */
int copyfile(unsigned int index)        {
    int h1,h2,i;
    char dest[80], fn[80], *buff;
    unsigned count = 0xff00;
    char *buf;

    h1 = open(filename[index],O_RDONLY | O_BINARY);
    if (h1 == -1) {
        strcpy(dest,"copyfile: Cannot open ");
        strcat(dest,filename[index]);
        fatal(dest); }

    /* Here is a hairy part: We need to trim the filename so that it
       no longer has any drive letter or directory specification.
       (So that we can use it on our destination filename, which doesn't
        need all that). I did that by searching the filename backwards
        until the '\' is found. Once found, I use the space to its right
    */

    i = strlen(filename[index]);
    while (i>-1) {
        if (filename[index][i] == '\\') break;
        i--;     }

    if (i>-1)
        strcpy(fn,&filename[index][i+1]);
    else
        strcpy(fn,filename[index]);

    /* Now we append the drive letter to our 'fn' variable.. */
    dest[0] = destination;
    dest[1] = 0;
    strcat(dest,":");
    strcat(dest,fn);

    /* Tell user about it, and open it for writing */
    printf("%s (%lu bytes)\n",filename[index],filesize[index]);
    h2 = open(dest,O_CREAT | O_BINARY | O_WRONLY, S_IREAD | S_IWRITE);
    if (h2 == -1) {
        strcat(dest," - file cannot be opened.");
        fatal(dest); }

    if (filesize[index] < count)
        count = (int)filesize[index];

    /* Try to allocate a good sized buffer (thanks Microsoft for the
       examples in your online help
    */

    if( (buf = (char *)malloc( (size_t)count )) == NULL ) {
        count = _memmax();
        if( (buf = (char *)malloc( (size_t)count )) == NULL )
            return ENOMEM; }

    /* Read-write until there's nothing left. */
    while( !eof( h1 ) ) {
        /* Read and write input. */
        if( (count = read( h1, buf, count )) == -1 )
            return errno;
        if( (count = write( h2, buf, count )) == - 1 )
            return errno; }

    /* Close files and release memory. */
    close( h1 );
    close( h2 );
    free( buf );
    return 0; /* signal that all went well */
}

/* here we load the filenames and sizes into filename and filesize arrayws
*/
unsigned load_file_info(int num,char *names[]) {
    int handle;
    int file=0;
    unsigned long totbytes=0L;
    unsigned long fsize;

    num -= 2;   /* we don't want the drive letter */
    for (;num>0;num--) {
        handle = open(names[num],O_BINARY | O_RDONLY);
        fsize = filelength(handle);
        close(handle);

        if (fsize<0L) {
            printf("Error learning size of %s - file ignored.\n",names[num]);
            continue; }

        if (fsize>3500000L) continue; /* probably a directory name */

        totbytes += fsize;
        /* dynamically create a string area to keep this filename 
        */
        filename[file] = malloc(sizeof(char) * strlen(names[num])+1);
        filesize[file] = fsize;

        if (filename[file] == NULL)
            fatal("Out of memory during load_file_info");

        strcpy(filename[file],names[num]);
        file++;
    }
    printf("%lu bytes in %u files.\n",totbytes,file);
    return(file);
}

/* Here we do two things: Learn the space left on disk from DOS (or OS/2) and
   we assign the 'disk_empty' variable, which we will use at pick_file later
   on. DOS and OS/2 gives us functions to find out the space left on disk, and
   the total size of that diskette.
*/

long learn_drive() {
#ifdef os2
    FSALLOCATE f; /* OS/2 headers use this structure definition */
#else
    struct diskfree_t space;
#endif
    unsigned int drivenum;
    long free,total;

    drivenum=destination - 'A' +1 ;
    disk_empty = 0;
#ifdef os2
    if (DosQFSInfo(drivenum,1,(char far *) &f,sizeof(f)) == 0) {
      free =  f.cbSector * f.cSectorUnit * f.cUnitAvail;
      total = f.cbSector * f.cSectorUnit * f.cUnit; }
    else /* some error occurred */
      free = -1;
#else
    _dos_getdiskfree(drivenum,&space);
    free = (long) space.avail_clusters * space.sectors_per_cluster * space.bytes_per_sector;
    total = (long) space.total_clusters * space.sectors_per_cluster * space.bytes_per_sector;
#endif
    disk_empty = (total - free) < 1024L;
    return (free);
}

void disp_help() {
    puts("FIT V1.1:(C)1990 Turgut Kalfaoglu,1378 Sok 8/10,Izmir 35210,Turkey\n");
    puts("FIT copies selected files onto diskettes so that they would take up the");
    puts("least number of diskettes.\n");
    puts("Usage:   FIT  wildcard_file_specs  destination_drive");
    puts("Example: FIT *.* \\*.COM A:\n");
    puts("This program is user-supported. If you find it useful, PLEASE");
    puts("send $10 to the above address. Thank you!");
    exit(1); }

/* displays error and quits */
void fatal(char *blurb) {
    puts(blurb);
    exit(2); }


